package javax.imageio.spi;

import java.util.ArrayList;
import java.util.ServiceLoader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class ServiceRegistry
{
    private Map categoryMap;
    
    public ServiceRegistry(Iterator<Class<?>> iterator) {
        this.categoryMap = new HashMap();
        if (iterator == null) {
            throw new IllegalArgumentException("categories == null!");
        }
        while (iterator.hasNext()) {
            Class<Object> clazz = (Class)iterator.next();
            this.categoryMap.put(clazz, new SubRegistry(this, clazz));
        }
        return;
    }
    
    public static <T> Iterator<T> lookupProviders(Class<T> service, ClassLoader loader) {
        if (service == null) {
            throw new IllegalArgumentException("providerClass == null!");
        }
        return ServiceLoader.load(service, loader).iterator();
    }
    
    public static <T> Iterator<T> lookupProviders(Class<T> service) {
        if (service == null) {
            throw new IllegalArgumentException("providerClass == null!");
        }
        return ServiceLoader.load(service).iterator();
    }
    
    public Iterator<Class<?>> getCategories() {
        return this.categoryMap.keySet().iterator();
    }
    
    private Iterator getSubRegistries(Object o) {
        ArrayList<Object> list = new ArrayList();
        for (Class clazz : this.categoryMap.keySet()) {
            if (clazz.isAssignableFrom(o.getClass())) {
                list.add((Object)this.categoryMap.get(clazz));
            }
            else {
                continue;
            }
        }
        return list.iterator();
    }
    
    public <T> boolean registerServiceProvider(T t, Class<T> clazz) {
        if (t == null) {
            throw new IllegalArgumentException("provider == null!");
        }
        SubRegistry subRegistry = (SubRegistry)this.categoryMap.get(clazz);
        if (subRegistry == null) {
            throw new IllegalArgumentException("category unknown!");
        }
        if (!clazz.isAssignableFrom(t.getClass())) {
            throw new ClassCastException();
        }
        return subRegistry.registerServiceProvider(t);
    }
    
    public void registerServiceProvider(Object o) {
        if (o == null) {
            throw new IllegalArgumentException("provider == null!");
        }
        Iterator<Object> subRegistries = this.getSubRegistries(o);
        while (subRegistries.hasNext()) {
            ((SubRegistry)subRegistries.next()).registerServiceProvider(o);
        }
        return;
    }
    
    public void registerServiceProviders(Iterator<?> iterator) {
        if (iterator == null) {
            throw new IllegalArgumentException("provider == null!");
        }
        while (iterator.hasNext()) {
            this.registerServiceProvider(iterator.next());
        }
        return;
    }
    
    public <T> boolean deregisterServiceProvider(T t, Class<T> clazz) {
        if (t == null) {
            throw new IllegalArgumentException("provider == null!");
        }
        SubRegistry subRegistry = (SubRegistry)this.categoryMap.get(clazz);
        if (subRegistry == null) {
            throw new IllegalArgumentException("category unknown!");
        }
        if (!clazz.isAssignableFrom(t.getClass())) {
            throw new ClassCastException();
        }
        return subRegistry.deregisterServiceProvider(t);
    }
    
    public void deregisterServiceProvider(Object o) {
        if (o == null) {
            throw new IllegalArgumentException("provider == null!");
        }
        Iterator<Object> subRegistries = this.getSubRegistries(o);
        while (subRegistries.hasNext()) {
            ((SubRegistry)subRegistries.next()).deregisterServiceProvider(o);
        }
        return;
    }
    
    public boolean contains(Object o) {
        if (o == null) {
            throw new IllegalArgumentException("provider == null!");
        }
        Iterator<Object> subRegistries = this.getSubRegistries(o);
        while (subRegistries.hasNext()) {
            if (((SubRegistry)subRegistries.next()).contains(o)) {
                return true;
            }
            continue;
        }
        return false;
    }
    
    public <T> Iterator<T> getServiceProviders(Class<T> clazz, boolean b) {
        SubRegistry subRegistry = (SubRegistry)this.categoryMap.get(clazz);
        if (subRegistry == null) {
            throw new IllegalArgumentException("category unknown!");
        }
        return subRegistry.getServiceProviders(b);
    }
    
    public <T> Iterator<T> getServiceProviders(Class<T> clazz, ServiceRegistry.Filter filter, boolean b) {
        if ((SubRegistry)this.categoryMap.get(clazz) == null) {
            throw new IllegalArgumentException("category unknown!");
        }
        return new FilterIterator(this.getServiceProviders(clazz, b), filter);
    }
    
    public <T> T getServiceProviderByClass(Class<T> clazz) {
        if (clazz == null) {
            throw new IllegalArgumentException("providerClass == null!");
        }
        for (Class clazz2 : this.categoryMap.keySet()) {
            if (clazz2.isAssignableFrom(clazz)) {
                Object serviceProviderByClass = ((SubRegistry)this.categoryMap.get(clazz2)).getServiceProviderByClass(clazz);
                if (serviceProviderByClass != null) {
                    return serviceProviderByClass;
                }
                continue;
            }
            else {
                continue;
            }
        }
        return null;
    }
    
    public <T> boolean setOrdering(Class<T> clazz, T t, T t2) {
        if (t == null || t2 == null) {
            throw new IllegalArgumentException("provider is null!");
        }
        if (t == t2) {
            throw new IllegalArgumentException("providers are the same!");
        }
        SubRegistry subRegistry = (SubRegistry)this.categoryMap.get(clazz);
        if (subRegistry == null) {
            throw new IllegalArgumentException("category unknown!");
        }
        return subRegistry.contains(t) && subRegistry.contains(t2) && subRegistry.setOrdering(t, t2);
    }
    
    public <T> boolean unsetOrdering(Class<T> clazz, T t, T t2) {
        if (t == null || t2 == null) {
            throw new IllegalArgumentException("provider is null!");
        }
        if (t == t2) {
            throw new IllegalArgumentException("providers are the same!");
        }
        SubRegistry subRegistry = (SubRegistry)this.categoryMap.get(clazz);
        if (subRegistry == null) {
            throw new IllegalArgumentException("category unknown!");
        }
        return subRegistry.contains(t) && subRegistry.contains(t2) && subRegistry.unsetOrdering(t, t2);
    }
    
    public void deregisterAll(Class<?> clazz) {
        SubRegistry subRegistry = (SubRegistry)this.categoryMap.get(clazz);
        if (subRegistry == null) {
            throw new IllegalArgumentException("category unknown!");
        }
        subRegistry.clear();
        return;
    }
    
    public void deregisterAll() {
        Iterator<Object> iterator = this.categoryMap.values().iterator();
        while (iterator.hasNext()) {
            ((SubRegistry)iterator.next()).clear();
        }
    }
    
    public void finalize() throws Throwable {
        this.deregisterAll();
        super.finalize();
    }
    
    public interface Filter
    {
        public abstract boolean filter(Object p0);
    }
}
